with ada.strings.unbounded;
use ada.strings.unbounded;
with Grammatiken; use Grammatiken;
with Baum; use Baum;

package Attributierungen is

   -- vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
   -- Da die konkreten Attribute, ihre Typen und die zugehrigen
   -- Auswertungsregeln stark von dem durch die Attributierung zu 
   -- lsenden Problem abhngen, ist der Inhalt dieser Datei eher
   -- als eine SCHABLONE zu verstehen. Insbesondere kann diese Da- 
   -- tei erst NACH den notwendigen Ergnzungen compiliert werden.
   -- ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

   -- Zur Modellierung von Attributierungen:
   -- --------------------------------------
   --
   -- Die Attributierung A ergnzt den in der Syntaxanalyse 
   -- erstellten Syntaxbaum B. Attribute sind den einzelnen
   -- Nichtterminalen Symbolen zugeordnet. Jedem Attribut A
   -- kann wie angedeutet die Information mitgegeben werden,
   -- ob A ererbt ist oder nicht - wird fr die systematische
   -- Erzeugung eines Attributauswerters bentigt.
   
   type SymbolTyp is Integer range 1 .. <anzNT>;
   
   type Attributierung_Rec (zu : SymbolTyp) is record 
      case zu is
         when 1 => -- Attribute des Startsymbols
            a11 : a11Typ; 
            -- evtl. ererbta11 : Boolean := False;           
            a12 : a12Typ; 
            -- evtl. ererbta12 : Boolean := False;
            ...
         when 2 => -- Attribute des nchsten NTs
            a21 : a21Typ;  
            -- evtl. ererbta21 : Boolean := False;           
            a22 : a22Typ; 
            -- evtl. ererbta22 : Boolean := False;
            ...
            
         ...   
         
         when <anzNT> => -- Attribute des letzten NTs
            a<anzNT>1 : a<anzNT>1Typ;
            -- evtl. ererbta<anzNT>1 : Boolean := False;            
            a<anzNT>2 : a<anzNT>2Typ;
            -- evtl. ererbta<anzNT>2 : Boolean := False;
            ...
      end case;
   end record; 

   type Attributierung is access Attributierung_Rec;
   
   
   -- Zur Modellierung attributierter Syntaxbume:
   -- --------------------------------------------
   --
   -- Die Attribute eines Syntaxbaums B lassen sich in einem 
   -- Array solcher Attributierungen unterbringen. 
   -- Dabei gilt:
   --   + Der Index ist die prae-Nummer des Knotens in B
   --   + Fr jeden Knoten K ist Attributierungs_Rec(K.code)
   --     der Typ von attribute(K.prae)

   type AttributArray is array (Integer range <>) 
      of Attributierung;
   
   attribute : AttributArray (1 .. <anzahlKnotenDesBaums>);   
   
   -- Das *ererbte* Attribut inh des Knotens K steht in 
   --    attribute(K.prae).inh und es gilt 
   --    attribute(K.prae).ererbtinh = True. 
   --
   -- Die zur Berechnung von inh bentigten Attribute findet
   -- man in attribute(K'.prae) fr K'-Knoten, die Geschwister
   -- oder der Vater von K sind.
   --
   -- Das *synthetisierte* Attribut syn des Knotens K steht in 
   --    attribute(K.prae).syn und es gilt 
   --    attribute(K.prae).ererbtsyn = False. 
   --
   -- Die zur Berechnung von syn bentigten Attribute findet
   -- man in attribute(K'.prae) fr K'-Knoten, die Kinder von
   -- K oder K selbst sind.
   --
   -- -------------------------------------------------------
   --
   -- Auf dieser Basis kann ein bedrafsgetriebener Auswerter
   -- (nach Jourdan) entweder von Hand geschrieben oder aus
   -- einer geeigneten Beschreibung generiert werden.
   
end Attributierungen;